@@ -43,6 +43,30 @@ WireId Arch::addWire(IdStringList name, IdString type, int x, int y)
     return wire;
 }
 
+WireId Arch::addWireAsBelInput(BelId bel, IdString name)
+{
+    Loc l = getBelLocation(bel);
+    WireId w = addWire(IdStringList::concat(getBelName(bel), name), name, l.x, l.y);
+    addBelInput(bel, name, w);
+    return w;
+}
+
+WireId Arch::addWireAsBelOutput(BelId bel, IdString name)
+{
+    Loc l = getBelLocation(bel);
+    WireId w = addWire(IdStringList::concat(getBelName(bel), name), name, l.x, l.y);
+    addBelOutput(bel, name, w);
+    return w;
+}
+
+WireId Arch::addWireAsBelInout(BelId bel, IdString name)
+{
+    Loc l = getBelLocation(bel);
+    WireId w = addWire(IdStringList::concat(getBelName(bel), name), name, l.x, l.y);
+    addBelInout(bel, name, w);
+    return w;
+}
+
 PipId Arch::addPip(IdStringList name, IdString type, WireId srcWire, WireId dstWire, delay_t delay, Loc loc)
 {
     NPNR_ASSERT(pip_by_name.count(name) == 0);
@@ -110,40 +134,20 @@ BelId Arch::addBel(IdStringList name, IdString type, Loc loc, bool gb, bool hidd
     return bel;
 }
 
-void Arch::addBelInput(BelId bel, IdString name, WireId wire)
+void Arch::addBelInput(BelId bel, IdString name, WireId wire) { addBelPin(bel, name, wire, PORT_IN); }
+
+void Arch::addBelOutput(BelId bel, IdString name, WireId wire) { addBelPin(bel, name, wire, PORT_OUT); }
+
+void Arch::addBelInout(BelId bel, IdString name, WireId wire) { addBelPin(bel, name, wire, PORT_INOUT); }
+
+void Arch::addBelPin(BelId bel, IdString name, WireId wire, PortType type)
 {
     auto &bi = bel_info(bel);
     NPNR_ASSERT(bi.pins.count(name) == 0);
     PinInfo &pi = bi.pins[name];
     pi.name = name;
     pi.wire = wire;
-    pi.type = PORT_IN;
-
-    if (wire != WireId())
-        wire_info(wire).bel_pins.push_back(BelPin{bel, name});
-}
-
-void Arch::addBelOutput(BelId bel, IdString name, WireId wire)
-{
-    auto &bi = bel_info(bel);
-    NPNR_ASSERT(bi.pins.count(name) == 0);
-    PinInfo &pi = bi.pins[name];
-    pi.name = name;
-    pi.wire = wire;
-    pi.type = PORT_OUT;
-
-    if (wire != WireId())
-        wire_info(wire).bel_pins.push_back(BelPin{bel, name});
-}
-
-void Arch::addBelInout(BelId bel, IdString name, WireId wire)
-{
-    auto &bi = bel_info(bel);
-    NPNR_ASSERT(bi.pins.count(name) == 0);
-    PinInfo &pi = bi.pins[name];
-    pi.name = name;
-    pi.wire = wire;
-    pi.type = PORT_INOUT;
+    pi.type = type;
 
     if (wire != WireId())
         wire_info(wire).bel_pins.push_back(BelPin{bel, name});
@@ -390,7 +394,8 @@ void Arch::unbindWire(WireId wire)
         refreshUiPip(pip);
     }
 
-    uarch->notifyWireChange(wire, nullptr);
+    if (uarch)
+        uarch->notifyWireChange(wire, nullptr);
     net_wires.erase(wire);
     wire_info(wire).bound_net = nullptr;
     refreshUiWire(wire);
@@ -580,7 +585,7 @@ bool Arch::place()
         bool have_iobuf_or_constr = false;
         for (auto &cell : cells) {
             CellInfo *ci = cell.second.get();
-            if (ci->type == id("GENERIC_IOB") || ci->bel != BelId() || ci->attrs.count(id("BEL"))) {
+            if (ci->isPseudo() || ci->type == id("GENERIC_IOB") || ci->bel != BelId() || ci->attrs.count(id("BEL"))) {
                 have_iobuf_or_constr = true;
                 break;
             }
