@@ -19,6 +19,7 @@
  */
 
 #include <boost/algorithm/string.hpp>
+#include <cells.h>
 #include <iostream>
 #include <math.h>
 #include <regex>
@@ -33,6 +34,8 @@
 
 NEXTPNR_NAMESPACE_BEGIN
 
+const PairPOD *pairLookup(const PairPOD *list, const size_t len, const int dest);
+
 // GUI
 void Arch::fixClockSpineDecals(void)
 {
@@ -175,6 +178,147 @@ DecalXY Arch::getWireDecal(WireId wire) const
     return wires.at(wire).decalxy_active;
 }
 
+bool Arch::allocate_longwire(NetInfo *ni, int lw_idx)
+{
+    NPNR_ASSERT(ni != nullptr);
+    if (ni->driver.cell == nullptr) {
+        return false;
+    }
+    if (ni->name == id("$PACKER_VCC_NET") || ni->name == id("$PACKER_GND_NET")) {
+        return false;
+    }
+    // So far only for OBUF
+    switch (ni->driver.cell->type.index) {
+    case ID_ODDR:  /* fall-through*/
+    case ID_ODDRC: /* fall-through*/
+    case ID_IOBUF: /* fall-through*/
+    case ID_TBUF:
+        return false;
+    case ID_OBUF:
+        if (getCtx()->debug) {
+            log_info("Long wire for IO %s\n", nameOf(ni));
+        }
+        ni = ni->driver.cell->ports.at(id_I).net;
+        return allocate_longwire(ni, lw_idx);
+        break;
+    default:
+        break;
+    }
+
+    if (getCtx()->debug) {
+        log_info("Requested index:%d\n", lw_idx);
+    }
+    if (avail_longwires == 0 || (lw_idx != -1 && (avail_longwires & (1 << lw_idx)) == 0)) {
+        return false;
+    }
+    int longwire = lw_idx;
+    if (lw_idx == -1) {
+        for (longwire = 7; longwire >= 0; --longwire) {
+            if (avail_longwires & (1 << longwire)) {
+                break;
+            }
+        }
+    }
+    avail_longwires &= ~(1 << longwire);
+
+    // BUFS cell
+    CellInfo *bufs;
+    char buf[40];
+    snprintf(buf, sizeof(buf), "$PACKER_BUFS%d", longwire);
+    std::unique_ptr<CellInfo> new_cell = create_generic_cell(getCtx(), id_BUFS, buf);
+    bufs = new_cell.get();
+    cells[bufs->name] = std::move(new_cell);
+    if (lw_idx != -1) {
+        bufs->cluster = bufs->name;
+        bufs->constr_z = lw_idx + BelZ::bufs_0_z;
+        bufs->constr_abs_z = true;
+        bufs->constr_children.clear();
+    }
+
+    // old driver -> bufs LW input net
+    snprintf(buf, sizeof(buf), "$PACKER_BUFS_%c", longwire + 'A');
+    auto net = std::make_unique<NetInfo>(id(buf));
+    NetInfo *bufs_net = net.get();
+    nets[net->name] = std::move(net);
+
+    // split the net
+    CellInfo *driver_cell = ni->driver.cell;
+    IdString driver_port = ni->driver.port;
+    driver_cell->disconnectPort(driver_port);
+
+    bufs->connectPort(id_O, ni);
+    bufs->connectPort(id_I, bufs_net);
+    driver_cell->connectPort(driver_port, bufs_net);
+
+    if (getCtx()->debug) {
+        log_info("Long wire %d was allocated\n", longwire);
+    }
+    return true;
+}
+
+void Arch::auto_longwires() {}
+
+void Arch::fix_longwire_bels()
+{
+    // After routing, it is clear which wires and in which bus SS00 and SS40 are used and
+    // in which quadrant they are routed. Here we write it in the attributes.
+    for (auto &cell : cells) {
+        CellInfo *ci = cell.second.get();
+        if (ci->type != id_BUFS) {
+            continue;
+        }
+        const NetInfo *ni = ci->getPort(id_O);
+        if (ni == nullptr) {
+            continue;
+        }
+        // bus wire is one of the wires
+        // value does not matter, but the L/R parameter itself
+        for (auto &wire : ni->wires) {
+            WireId w = wires[wire.first].type;
+            switch (w.hash()) {
+            case ID_LWSPINETL0:
+            case ID_LWSPINETL1:
+            case ID_LWSPINETL2:
+            case ID_LWSPINETL3:
+            case ID_LWSPINETL4:
+            case ID_LWSPINETL5:
+            case ID_LWSPINETL6:
+            case ID_LWSPINETL7:
+            case ID_LWSPINEBL0:
+            case ID_LWSPINEBL1:
+            case ID_LWSPINEBL2:
+            case ID_LWSPINEBL3:
+            case ID_LWSPINEBL4:
+            case ID_LWSPINEBL5:
+            case ID_LWSPINEBL6:
+            case ID_LWSPINEBL7:
+                ci->setParam(id("L"), Property(w.str(this)));
+                break;
+            case ID_LWSPINETR0:
+            case ID_LWSPINETR1:
+            case ID_LWSPINETR2:
+            case ID_LWSPINETR3:
+            case ID_LWSPINETR4:
+            case ID_LWSPINETR5:
+            case ID_LWSPINETR6:
+            case ID_LWSPINETR7:
+            case ID_LWSPINEBR0:
+            case ID_LWSPINEBR1:
+            case ID_LWSPINEBR2:
+            case ID_LWSPINEBR3:
+            case ID_LWSPINEBR4:
+            case ID_LWSPINEBR5:
+            case ID_LWSPINEBR6:
+            case ID_LWSPINEBR7:
+                ci->setParam(id("R"), Property(w.str(this)));
+                break;
+            default:
+                break;
+            }
+        }
+    }
+}
+
 WireInfo &Arch::wire_info(IdString wire)
 {
     auto w = wires.find(wire);
@@ -199,6 +343,14 @@ BelInfo &Arch::bel_info(IdString bel)
     return b->second;
 }
 
+NetInfo &Arch::net_info(IdString net)
+{
+    auto b = nets.find(net);
+    if (b == nets.end())
+        NPNR_ASSERT_FALSE_STR("no net named " + net.str(this));
+    return *b->second;
+}
+
 void Arch::addWire(IdString name, IdString type, int x, int y)
 {
     NPNR_ASSERT(wires.count(name) == 0);
@@ -401,6 +553,11 @@ void Arch::setDelayScaling(double scale, double offset)
     args.delayOffset = offset;
 }
 
+void Arch::addCellTimingClass(IdString cell, IdString port, TimingPortClass cls)
+{
+    cellTiming[cell].portClasses[port] = cls;
+}
+
 void Arch::addCellTimingClock(IdString cell, IdString port) { cellTiming[cell].portClasses[port] = TMG_CLOCK_INPUT; }
 
 void Arch::addCellTimingDelay(IdString cell, IdString fromPort, IdString toPort, DelayQuad delay)
@@ -487,7 +644,15 @@ IdString Arch::wireToGlobal(int &row, int &col, const DatabasePOD *db, IdString
     }
     snprintf(buf, 32, "%c%d0", direction, num);
     wire = id(buf);
-    snprintf(buf, 32, "R%dC%d_%c%d", row + 1, col + 1, direction, num);
+    // local aliases
+    const TilePOD *tile = db->grid[row * db->cols + col].get();
+    auto local_alias = pairLookup(tile->aliases.get(), tile->num_aliases, wire.index);
+    if (local_alias != nullptr) {
+        wire = IdString(local_alias->src_id);
+        snprintf(buf, 32, "R%dC%d_%s", row + 1, col + 1, wire.c_str(this));
+    } else {
+        snprintf(buf, 32, "R%dC%d_%c%d", row + 1, col + 1, direction, num);
+    }
     return id(buf);
 }
 
@@ -507,6 +672,7 @@ bool aliasCompare(GlobalAliasPOD i, GlobalAliasPOD j)
     return (i.dest_row < j.dest_row) || (i.dest_row == j.dest_row && i.dest_col < j.dest_col) ||
            (i.dest_row == j.dest_row && i.dest_col == j.dest_col && i.dest_id < j.dest_id);
 }
+
 bool timingCompare(TimingPOD i, TimingPOD j) { return i.name_id < j.name_id; }
 
 template <class T, class C> const T *genericLookup(const T *first, int len, const T val, C compare)
@@ -628,6 +794,28 @@ DelayQuad Arch::getWireTypeDelay(IdString wire)
     case ID_W830:
         len = id_X8;
         break;
+    case ID_LT02:
+    case ID_LT13:
+        glbsrc = id_SPINE_TAP_SCLK_0;
+        break;
+    case ID_LT01:
+    case ID_LT04:
+        glbsrc = id_SPINE_TAP_SCLK_1;
+        break;
+    case ID_LBO0:
+    case ID_LBO1:
+        glbsrc = id_TAP_BRANCH_SCLK;
+        break;
+    case ID_LB01:
+    case ID_LB11:
+    case ID_LB21:
+    case ID_LB31:
+    case ID_LB41:
+    case ID_LB51:
+    case ID_LB61:
+    case ID_LB71:
+        glbsrc = id_BRANCH_SCLK;
+        break;
     case ID_GT00:
     case ID_GT10:
         glbsrc = id_SPINE_TAP_PCLK;
@@ -647,7 +835,9 @@ DelayQuad Arch::getWireTypeDelay(IdString wire)
         glbsrc = id_BRANCH_PCLK;
         break;
     default:
-        if (wire.str(this).rfind("SPINE", 0) == 0) {
+        if (wire.str(this).rfind("LWSPINE", 0) == 0) {
+            glbsrc = IdString(ID_CENT_SPINE_SCLK);
+        } else if (wire.str(this).rfind("SPINE", 0) == 0) {
             glbsrc = IdString(ID_CENT_SPINE_PCLK);
         } else if (wire.str(this).rfind("UNK", 0) == 0) {
             glbsrc = IdString(ID_PIO_CENT_PCLK);
@@ -691,13 +881,15 @@ void Arch::read_cst(std::istream &in)
     std::regex port_attrre = std::regex("([^ =;]+=[^ =;]+) *([^;]*;)");
     std::regex iobelre = std::regex("IO([TRBL])([0-9]+)\\[?([A-Z])\\]?");
     std::regex inslocre = std::regex("INS_LOC +\"([^\"]+)\" +R([0-9]+)C([0-9]+)\\[([0-9])\\]\\[([AB])\\] *;.*");
+    std::regex clockre = std::regex("CLOCK_LOC +\"([^\"]+)\" +BUF([GS])(\\[([0-7])\\])?[^;]*;.*");
     std::smatch match, match_attr, match_pinloc;
     std::string line, pinline;
     enum
     {
         ioloc,
         ioport,
-        insloc
+        insloc,
+        clock
     } cst_type;
 
     settings.erase(id_cst);
@@ -708,24 +900,50 @@ void Arch::read_cst(std::istream &in)
             if (std::regex_match(line, match, portre)) {
                 cst_type = ioport;
             } else {
-                if (std::regex_match(line, match, inslocre)) {
-                    cst_type = insloc;
+                if (std::regex_match(line, match, clockre)) {
+                    cst_type = clock;
                 } else {
-                    if ((!line.empty()) && (line.rfind("//", 0) == std::string::npos)) {
-                        log_warning("Invalid constraint: %s\n", line.c_str());
+                    if (std::regex_match(line, match, inslocre)) {
+                        cst_type = insloc;
+                    } else {
+                        if ((!line.empty()) && (line.rfind("//", 0) == std::string::npos)) {
+                            log_warning("Invalid constraint: %s\n", line.c_str());
+                        }
+                        continue;
                     }
-                    continue;
                 }
             }
         }
 
         IdString net = id(match[1]);
         auto it = cells.find(net);
-        if (it == cells.end()) {
+        if (cst_type != clock && it == cells.end()) {
             log_info("Cell %s not found\n", net.c_str(this));
             continue;
         }
         switch (cst_type) {
+        case clock: { // CLOCK name BUFG|S=#
+            std::string which_clock = match[2];
+            std::string lw = match[4];
+            int lw_idx = -1;
+            if (lw.length() > 0) {
+                lw_idx = atoi(lw.c_str());
+                log_info("lw_idx:%d\n", lw_idx);
+            }
+            if (which_clock.at(0) == 'S') {
+                auto ni = nets.find(net);
+                if (ni == nets.end()) {
+                    log_info("Net %s not found\n", net.c_str(this));
+                    continue;
+                }
+                if (!allocate_longwire(ni->second.get(), lw_idx)) {
+                    log_info("Can't use the long wires. The %s network will use normal routing.\n", net.c_str(this));
+                }
+            } else {
+                log_info("BUFG isn't supported\n");
+                continue;
+            }
+        } break;
         case ioloc: { // IO_LOC name pin
             IdString pinname = id(match[2]);
             pinline = match[2];
@@ -804,7 +1022,7 @@ void Arch::addMuxBels(const DatabasePOD *db, int row, int col)
         // bel
         snprintf(buf, 40, "R%dC%d_MUX2_LUT%c%c", grow, gcol, mux_names[j].type, mux_names[j].bel_idx);
         belname = id(buf);
-        snprintf(buf, 40, "GW_MUX2_LUT%c", mux_names[j].type);
+        snprintf(buf, 40, "MUX2_LUT%c", mux_names[j].type);
         bel_id = id(buf);
         addBel(belname, bel_id, Loc(col, row, z), false);
 
@@ -950,6 +1168,8 @@ Arch::Arch(ArchArgs args) : args(args)
     log_info("Series:%s Device:%s Package:%s Speed:%s\n", family.c_str(), device_id.c_str(this),
              package_name.c_str(this), speed_id.c_str(this));
 
+    device = device_id.str(this);
+
     // setup db
     // add global VCC and GND bels
     addBel(id_GND, id_GND, Loc(0, 0, BelZ::gnd_0_z), true);
@@ -1001,6 +1221,31 @@ Arch::Arch(ArchArgs args) : args(args)
             bool dff = true;
             bool oddrc = false;
             switch (static_cast<ConstIds>(bel->type_id)) {
+            case ID_BUFS7:
+                z++; /* fall-through*/
+            case ID_BUFS6:
+                z++; /* fall-through*/
+            case ID_BUFS5:
+                z++; /* fall-through*/
+            case ID_BUFS4:
+                z++; /* fall-through*/
+            case ID_BUFS3:
+                z++; /* fall-through*/
+            case ID_BUFS2:
+                z++; /* fall-through*/
+            case ID_BUFS1:
+                z++; /* fall-through*/
+            case ID_BUFS0:
+                snprintf(buf, 32, "R%dC%d_BUFS%d", row + 1, col + 1, z);
+                belname = id(buf);
+                addBel(belname, id_BUFS, Loc(col, row, BelZ::bufs_0_z + z), false);
+                portname = IdString(pairLookup(bel->ports.get(), bel->num_ports, ID_I)->src_id);
+                snprintf(buf, 32, "R%dC%d_%s", row + 1, col + 1, portname.c_str(this));
+                addBelInput(belname, id_I, id(buf));
+                portname = IdString(pairLookup(bel->ports.get(), bel->num_ports, ID_O)->src_id);
+                snprintf(buf, 32, "R%dC%d_%s", row + 1, col + 1, portname.c_str(this));
+                addBelOutput(belname, id_O, id(buf));
+                break;
             case ID_GSR0:
                 snprintf(buf, 32, "R%dC%d_GSR0", row + 1, col + 1);
                 belname = id(buf);
@@ -1047,6 +1292,36 @@ Arch::Arch(ArchArgs args) : args(args)
                 snprintf(buf, 32, "R%dC%d_%s", row + 1, col + 1, portname.c_str(this));
                 addBelInput(belname, id_OSCEN, id(buf));
                 break;
+            case ID_RAM16:
+                snprintf(buf, 32, "R%dC%d_RAMW", row + 1, col + 1);
+                belname = id(buf);
+                addBel(belname, id_RAMW, Loc(col, row, BelZ::lutram_0_z), false);
+
+                snprintf(buf, 32, "R%dC%d_A%d", row + 1, col + 1, 4);
+                addBelInput(belname, id_A4, id(buf));
+                snprintf(buf, 32, "R%dC%d_B%d", row + 1, col + 1, 4);
+                addBelInput(belname, id_B4, id(buf));
+                snprintf(buf, 32, "R%dC%d_C%d", row + 1, col + 1, 4);
+                addBelInput(belname, id_C4, id(buf));
+                snprintf(buf, 32, "R%dC%d_D%d", row + 1, col + 1, 4);
+                addBelInput(belname, id_D4, id(buf));
+
+                snprintf(buf, 32, "R%dC%d_A%d", row + 1, col + 1, 5);
+                addBelInput(belname, id_A5, id(buf));
+                snprintf(buf, 32, "R%dC%d_B%d", row + 1, col + 1, 5);
+                addBelInput(belname, id_B5, id(buf));
+                snprintf(buf, 32, "R%dC%d_C%d", row + 1, col + 1, 5);
+                addBelInput(belname, id_C5, id(buf));
+                snprintf(buf, 32, "R%dC%d_D%d", row + 1, col + 1, 5);
+                addBelInput(belname, id_D5, id(buf));
+
+                snprintf(buf, 32, "R%dC%d_CLK%d", row + 1, col + 1, 2);
+                addBelInput(belname, id_CLK, id(buf));
+                snprintf(buf, 32, "R%dC%d_LSR%d", row + 1, col + 1, 2);
+                addBelInput(belname, id_LSR, id(buf));
+                snprintf(buf, 32, "R%dC%d_CE%d", row + 1, col + 1, 2);
+                addBelInput(belname, id_CE, id(buf));
+                break;
             // fall through the ++
             case ID_LUT7:
                 z++;
@@ -1274,12 +1549,6 @@ Arch::Arch(ArchArgs args) : args(args)
                 snprintf(buf, 32, "R%dC%d_%s_%s", row + 1, col + 1, srcid.c_str(this), destid.c_str(this));
                 IdString pipname = id(buf);
                 DelayQuad delay = getWireTypeDelay(destid);
-                // local alias
-                auto local_alias = pairLookup(tile->aliases.get(), tile->num_aliases, srcid.index);
-                if (local_alias != nullptr) {
-                    srcid = IdString(local_alias->src_id);
-                    gsrcname = wireToGlobal(srcrow, srccol, db, srcid);
-                }
                 // global alias
                 srcid = IdString(pip.src_id);
                 GlobalAliasPOD alias;
@@ -1644,6 +1913,11 @@ bool Arch::place()
 bool Arch::route()
 {
     std::string router = str_or_default(settings, id_router, defaultRouter);
+
+    if (bool_or_default(settings, id("arch.enable-globals"))) {
+        route_gowin_globals(getCtx());
+    }
+
     bool result;
     if (router == "router1") {
         result = router1(getCtx(), Router1Cfg(getCtx()));
@@ -1655,6 +1929,7 @@ bool Arch::route()
     }
     getCtx()->settings[id_route] = 1;
     archInfoToAttributes();
+    fix_longwire_bels();
     return result;
 }
 
@@ -1752,6 +2027,8 @@ void Arch::assignArchInfo()
 
             // add timing paths
             addCellTimingClock(cname, id_CLK);
+            addCellTimingClass(cname, id_CE, TMG_REGISTER_INPUT);
+            addCellTimingClass(cname, id_LSR, TMG_REGISTER_INPUT);
             IdString ports[4] = {id_A, id_B, id_C, id_D};
             for (int i = 0; i < 4; i++) {
                 DelayPair setup =
@@ -1769,20 +2046,32 @@ void Arch::assignArchInfo()
             }
             break;
         }
-        case ID_GW_MUX2_LUT8:
+        case ID_MUX2_LUT8:
             delay = delay + delayLookup(speed->lut.timings.get(), speed->lut.num_timings, id_fx_ofx1);
             /* FALLTHRU */
-        case ID_GW_MUX2_LUT7:
+        case ID_MUX2_LUT7:
             delay = delay + delayLookup(speed->lut.timings.get(), speed->lut.num_timings, id_fx_ofx1);
             /* FALLTHRU */
-        case ID_GW_MUX2_LUT6:
+        case ID_MUX2_LUT6:
             delay = delay + delayLookup(speed->lut.timings.get(), speed->lut.num_timings, id_fx_ofx1);
             /* FALLTHRU */
-        case ID_GW_MUX2_LUT5: {
+        case ID_MUX2_LUT5: {
             delay = delay + delayLookup(speed->lut.timings.get(), speed->lut.num_timings, id_fx_ofx1);
             addCellTimingDelay(cname, id_I0, id_OF, delay);
             addCellTimingDelay(cname, id_I1, id_OF, delay);
+            addCellTimingClass(cname, id_SEL, TMG_COMB_INPUT);
+            break;
         }
+        case ID_IOB:
+            /* FALLTHRU */
+        case ID_IOBS:
+            addCellTimingClass(cname, id_I, TMG_ENDPOINT);
+            addCellTimingClass(cname, id_O, TMG_STARTPOINT);
+            break;
+        case ID_BUFS:
+            addCellTimingClass(cname, id_I, TMG_ENDPOINT);
+            addCellTimingClass(cname, id_O, TMG_STARTPOINT);
+            break;
         default:
             break;
         }
@@ -1826,4 +2115,22 @@ bool Arch::cellsCompatible(const CellInfo **cells, int count) const
     return true;
 }
 
+void Arch::route_gowin_globals(Context *ctx) { globals_router.route_globals(ctx); }
+
+void Arch::mark_gowin_globals(Context *ctx) { globals_router.mark_globals(ctx); }
+// ---------------------------------------------------------------
+void Arch::pre_pack(Context *ctx)
+{
+    if (bool_or_default(settings, id("arch.enable-auto-longwires"))) {
+        auto_longwires();
+    }
+}
+
+void Arch::post_pack(Context *ctx)
+{
+    if (bool_or_default(settings, id("arch.enable-globals"))) {
+        mark_gowin_globals(ctx);
+    }
+}
+
 NEXTPNR_NAMESPACE_END
