@@ -31,6 +31,8 @@
 #include "nextpnr_namespaces.h"
 #include "nextpnr_types.h"
 
+#include "globals.h"
+
 NEXTPNR_NAMESPACE_BEGIN
 
 template <typename T> struct RelPtr
@@ -289,6 +291,7 @@ struct Arch : BaseArch<ArchRanges>
     WireInfo &wire_info(IdString wire);
     PipInfo &pip_info(IdString pip);
     BelInfo &bel_info(IdString bel);
+    NetInfo &net_info(IdString net);
 
     std::vector<IdString> bel_ids, wire_ids, pip_ids;
 
@@ -336,6 +339,7 @@ struct Arch : BaseArch<ArchRanges>
     void setDelayScaling(double scale, double offset);
 
     void addCellTimingClock(IdString cell, IdString port);
+    void addCellTimingClass(IdString cell, IdString port, TimingPortClass cls);
     void addCellTimingDelay(IdString cell, IdString fromPort, IdString toPort, DelayQuad delay);
     void addCellTimingSetupHold(IdString cell, IdString port, IdString clock, DelayPair setup, DelayPair hold);
     void addCellTimingClockToOut(IdString cell, IdString port, IdString clock, DelayQuad clktoq);
@@ -459,6 +463,16 @@ struct Arch : BaseArch<ArchRanges>
     void assignArchInfo() override;
     bool cellsCompatible(const CellInfo **cells, int count) const;
     bool haveBelType(int x, int y, IdString bel_type);
+    bool allocate_longwire(NetInfo *ni, int lw_idx = -1);
+    void fix_longwire_bels();
+    void pre_pack(Context *ctx);
+    void post_pack(Context *ctx);
+    void auto_longwires();
+
+    GowinGlobalRouter globals_router;
+    void mark_gowin_globals(Context *ctx);
+    void route_gowin_globals(Context *ctx);
+
     // chip db version
     unsigned int const chipdb_version = 1;
 
@@ -475,6 +489,9 @@ struct Arch : BaseArch<ArchRanges>
     // XXX GW1NR-9 iobuf quirk
     bool gw1n9_quirk = false;
 
+    // 8 Long wires
+    uint8_t avail_longwires = 0xff;
+
     // Permissible combinations of modes in a single slice
     std::map<const IdString, IdString> dff_comp_mode;
 };
@@ -485,9 +502,12 @@ enum
 {
     mux_0_z = 10,     // start Z for the MUX2LUT5 bels
     iologic_0_z = 20, // start Z for the IOLOGIC bels
+    lutram_0_z = 30,  // start Z for the IOLOGIC bels
     vcc_0_z = 277,    // virtual VCC bel Z
     gnd_0_z = 278,    // virtual VSS bel Z
-    osc_z = 280       // Z for the oscillator bels
+    osc_z = 280,      // Z for the oscillator bels
+    bufs_0_z = 281,   // Z for long wire buffer bel
+    free_z = 289      // Must be the last, one can use z starting from this value, adjust accordingly.
 };
 }
 
