@@ -26,6 +26,8 @@
 #include "log.h"
 #include "util.h"
 
+#include "globals.h"
+
 NEXTPNR_NAMESPACE_BEGIN
 
 static void make_dummy_alu(Context *ctx, int alu_idx, CellInfo *ci, CellInfo *packed_head,
@@ -264,7 +266,7 @@ static void pack_mux2_lut5(Context *ctx, CellInfo *ci, pool<IdString> &packed_ce
             return;
         }
 
-        std::unique_ptr<CellInfo> packed = create_generic_cell(ctx, id_GW_MUX2_LUT5, ci->name.str(ctx) + "_LC");
+        std::unique_ptr<CellInfo> packed = create_generic_cell(ctx, id_MUX2_LUT5, ci->name.str(ctx) + "_LC");
         if (ctx->verbose) {
             log_info("packed cell %s into %s\n", ctx->nameOf(ci), ctx->nameOf(packed.get()));
         }
@@ -307,7 +309,7 @@ static void pack_mux2_lut5(Context *ctx, CellInfo *ci, pool<IdString> &packed_ce
             return;
         }
 
-        std::unique_ptr<CellInfo> packed = create_generic_cell(ctx, id_GW_MUX2_LUT5, ci->name.str(ctx) + "_LC");
+        std::unique_ptr<CellInfo> packed = create_generic_cell(ctx, id_MUX2_LUT5, ci->name.str(ctx) + "_LC");
         if (ctx->verbose) {
             log_info("packed cell %s into %s\n", ctx->nameOf(ci), ctx->nameOf(packed.get()));
         }
@@ -411,7 +413,7 @@ static void pack_mux2_lut6(Context *ctx, CellInfo *ci, pool<IdString> &packed_ce
 {
     static int x[] = {0, 0};
     static int z[] = {+1, -1};
-    pack_mux2_lut(ctx, ci, is_gw_mux2_lut5, '6', id_GW_MUX2_LUT6, x, z, packed_cells, delete_nets, new_cells);
+    pack_mux2_lut(ctx, ci, is_mux2_lut5, '6', id_MUX2_LUT6, x, z, packed_cells, delete_nets, new_cells);
 }
 
 // pack MUX2_LUT7
@@ -420,7 +422,7 @@ static void pack_mux2_lut7(Context *ctx, CellInfo *ci, pool<IdString> &packed_ce
 {
     static int x[] = {0, 0};
     static int z[] = {+2, -2};
-    pack_mux2_lut(ctx, ci, is_gw_mux2_lut6, '7', id_GW_MUX2_LUT7, x, z, packed_cells, delete_nets, new_cells);
+    pack_mux2_lut(ctx, ci, is_mux2_lut6, '7', id_MUX2_LUT7, x, z, packed_cells, delete_nets, new_cells);
 }
 
 // pack MUX2_LUT8
@@ -429,7 +431,7 @@ static void pack_mux2_lut8(Context *ctx, CellInfo *ci, pool<IdString> &packed_ce
 {
     static int x[] = {1, 0};
     static int z[] = {-4, -4};
-    pack_mux2_lut(ctx, ci, is_gw_mux2_lut7, '8', id_GW_MUX2_LUT8, x, z, packed_cells, delete_nets, new_cells);
+    pack_mux2_lut(ctx, ci, is_mux2_lut7, '8', id_MUX2_LUT8, x, z, packed_cells, delete_nets, new_cells);
 }
 
 // Pack wide LUTs
@@ -692,6 +694,92 @@ static void pack_gsr(Context *ctx)
     }
 }
 
+// Pack shadow RAM
+void pack_sram(Context *ctx)
+{
+    log_info("Packing Shadow RAM..\n");
+
+    pool<IdString> packed_cells;
+    std::vector<std::unique_ptr<CellInfo>> new_cells;
+
+    for (auto &cell : ctx->cells) {
+        CellInfo *ci = cell.second.get();
+        if (is_sram(ctx, ci)) {
+
+            // Create RAMW slice
+            std::unique_ptr<CellInfo> ramw_slice = create_generic_cell(ctx, id_RAMW, ci->name.str(ctx) + "$RAMW_SLICE");
+            sram_to_ramw_split(ctx, ci, ramw_slice.get());
+            ramw_slice->connectPort(id_CE, ctx->nets[ctx->id("$PACKER_VCC_NET")].get());
+
+            // Create actual RAM slices
+            std::unique_ptr<CellInfo> ram_comb[4];
+            for (int i = 0; i < 4; i++) {
+                ram_comb[i] = create_generic_cell(ctx, id_SLICE, ci->name.str(ctx) + "$SRAM_SLICE" + std::to_string(i));
+                ram_comb[i]->params[id_FF_USED] = 1;
+                ram_comb[i]->params[id_FF_TYPE] = std::string("RAM");
+                sram_to_slice(ctx, ci, ram_comb[i].get(), i);
+            }
+            // Create 'block' SLICEs as a placement hint that these cells are mutually exclusive with the RAMW
+            std::unique_ptr<CellInfo> ramw_block[2];
+            for (int i = 0; i < 2; i++) {
+                ramw_block[i] =
+                        create_generic_cell(ctx, id_SLICE, ci->name.str(ctx) + "$RAMW_BLOCK" + std::to_string(i));
+                ram_comb[i]->params[id_FF_USED] = 1;
+                ramw_block[i]->params[id_FF_TYPE] = std::string("RAM");
+            }
+
+            // Disconnect ports of original cell after packing
+            // ci->disconnectPort(id_WCK);
+            // ci->disconnectPort(id_WRE);
+
+            for (int i = 0; i < 4; i++)
+                ci->disconnectPort(ctx->idf("RAD[%d]", i));
+
+            // Setup placement constraints
+            // Use the 0th bit as an anchor
+            ram_comb[0]->constr_abs_z = true;
+            ram_comb[0]->constr_z = 0;
+            ram_comb[0]->cluster = ram_comb[0]->name;
+            for (int i = 1; i < 4; i++) {
+                ram_comb[i]->cluster = ram_comb[0]->name;
+                ram_comb[i]->constr_abs_z = true;
+                ram_comb[i]->constr_x = 0;
+                ram_comb[i]->constr_y = 0;
+                ram_comb[i]->constr_z = i;
+                ram_comb[0]->constr_children.push_back(ram_comb[i].get());
+            }
+            for (int i = 0; i < 2; i++) {
+                ramw_block[i]->cluster = ram_comb[0]->name;
+                ramw_block[i]->constr_abs_z = true;
+                ramw_block[i]->constr_x = 0;
+                ramw_block[i]->constr_y = 0;
+                ramw_block[i]->constr_z = i + 4;
+                ram_comb[0]->constr_children.push_back(ramw_block[i].get());
+            }
+
+            ramw_slice->cluster = ram_comb[0]->name;
+            ramw_slice->constr_abs_z = true;
+            ramw_slice->constr_x = 0;
+            ramw_slice->constr_y = 0;
+            ramw_slice->constr_z = BelZ::lutram_0_z;
+            ram_comb[0]->constr_children.push_back(ramw_slice.get());
+
+            for (int i = 0; i < 4; i++)
+                new_cells.push_back(std::move(ram_comb[i]));
+            for (int i = 0; i < 2; i++)
+                new_cells.push_back(std::move(ramw_block[i]));
+            new_cells.push_back(std::move(ramw_slice));
+            packed_cells.insert(ci->name);
+        }
+    }
+    for (auto pcell : packed_cells) {
+        ctx->cells.erase(pcell);
+    }
+    for (auto &ncell : new_cells) {
+        ctx->cells[ncell->name] = std::move(ncell);
+    }
+}
+
 static bool is_nextpnr_iob(const Context *ctx, CellInfo *cell)
 {
     return cell->type == ctx->id("$nextpnr_ibuf") || cell->type == ctx->id("$nextpnr_obuf") ||
@@ -947,9 +1035,11 @@ static void pack_io(Context *ctx)
             if (constr_bel != ci->attrs.end()) {
                 constr_bel_name = constr_bel->second.as_string();
             }
-            constr_bel = iob->attrs.find(id_BEL);
-            if (constr_bel != iob->attrs.end()) {
-                constr_bel_name = constr_bel->second.as_string();
+            if (iob != nullptr) {
+                constr_bel = iob->attrs.find(id_BEL);
+                if (constr_bel != iob->attrs.end()) {
+                    constr_bel_name = constr_bel->second.as_string();
+                }
             }
             if (!constr_bel_name.empty()) {
                 BelId constr_bel = ctx->getBelByNameStr(constr_bel_name);
@@ -1007,7 +1097,9 @@ bool Arch::pack()
     Context *ctx = getCtx();
     try {
         log_break();
+        pre_pack(ctx);
         pack_constants(ctx);
+        pack_sram(ctx);
         pack_gsr(ctx);
         pack_io(ctx);
         pack_diff_io(ctx);
@@ -1016,6 +1108,7 @@ bool Arch::pack()
         pack_alus(ctx);
         pack_lut_lutffs(ctx);
         pack_nonlut_ffs(ctx);
+        post_pack(ctx);
         ctx->settings[id_pack] = 1;
         ctx->assignArchInfo();
         log_info("Checksum: 0x%08x\n", ctx->checksum());
