@@ -620,9 +620,13 @@ void write_asc(const Context *ctx, std::ostream &out)
             bool negclk_w = get_param_or_def(ctx, cell.second.get(), id_NEG_CLK_W);
             int write_mode = get_param_or_def(ctx, cell.second.get(), id_WRITE_MODE);
             int read_mode = get_param_or_def(ctx, cell.second.get(), id_READ_MODE);
-            set_config(ti_ramb, config.at(y).at(x), "NegClk", negclk_w);
-            set_config(ti_ramt, config.at(y + 1).at(x), "NegClk", negclk_r);
-
+            if (ctx->args.type == ArchArgs::UP5K || ctx->args.type == ArchArgs::UP3K) {
+                set_config(ti_ramb, config.at(y).at(x), "NegClk", negclk_r);
+                set_config(ti_ramt, config.at(y + 1).at(x), "NegClk", negclk_w);
+            } else {
+                set_config(ti_ramb, config.at(y).at(x), "NegClk", negclk_w);
+                set_config(ti_ramt, config.at(y + 1).at(x), "NegClk", negclk_r);
+            }
             set_config(ti_ramt, config.at(y + 1).at(x), "RamConfig.CBIT_0", write_mode & 0x1);
             set_config(ti_ramt, config.at(y + 1).at(x), "RamConfig.CBIT_1", write_mode & 0x2);
             set_config(ti_ramt, config.at(y + 1).at(x), "RamConfig.CBIT_2", read_mode & 0x1);
@@ -640,8 +644,7 @@ void write_asc(const Context *ctx, std::ostream &out)
                     {"CURRENT_MODE", 1}, {"RGB0_CURRENT", 6}, {"RGB1_CURRENT", 6}, {"RGB2_CURRENT", 6}};
             configure_extra_cell(config, ctx, cell.second.get(), rgba_params, true, std::string("IpConfig."));
             set_ec_cbit(config, ctx, get_ec_config(ctx->chip_info, cell.second->bel), "RGBA_DRV_EN", true, "IpConfig.");
-        } else if (cell.second->type == id_SB_WARMBOOT || cell.second->type == id_ICESTORM_LFOSC ||
-                   cell.second->type == id_SB_LEDDA_IP) {
+        } else if (cell.second->type.in(id_SB_WARMBOOT, id_ICESTORM_LFOSC, id_SB_LEDDA_IP)) {
             // No config needed
         } else if (cell.second->type == id_SB_I2C) {
             bool sda_in_dly = !cell.second->attrs.count(id_SDA_INPUT_DELAYED) ||
