@@ -435,7 +435,7 @@ struct Arch : BaseArch<ArchRanges>
     // Extra helper
     std::string get_full_chip_name() const;
 
-    IdString archId() const override { return id("machxo2"); }
+    IdString archId() const override { return id_machxo2; }
     ArchArgs archArgs() const override { return args; }
     IdString archArgsToId(ArchArgs args) const override;
 
@@ -626,7 +626,7 @@ struct Arch : BaseArch<ArchRanges>
 
     // Delay
     delay_t estimateDelay(WireId src, WireId dst) const override;
-    delay_t predictDelay(const NetInfo *net_info, const PortRef &sink) const override;
+    delay_t predictDelay(BelId src_bel, IdString src_pin, BelId dst_bel, IdString dst_pin) const override;
     delay_t getDelayEpsilon() const override { return 0.001; }
     delay_t getRipupDelayPenalty() const override { return 0.015; }
     float getDelayNS(delay_t v) const override { return v; }
