@@ -29,129 +29,41 @@ struct MistralBitgen
     Context *ctx;
     CycloneV *cv;
 
-    void init()
+    using rnode_t = CycloneV::rnode_t;
+    using pnode_t = CycloneV::pnode_t;
+    using pos_t = CycloneV::pos_t;
+    using block_type_t = CycloneV::block_type_t;
+    using port_type_t = CycloneV::port_type_t;
+
+    rnode_t find_rnode(block_type_t bt, pos_t pos, port_type_t port, int bi = -1, int pi = -1) const
     {
-        ctx->init_base_bitstream();
-        // Default options
-        cv->opt_b_set(CycloneV::ALLOW_DEVICE_WIDE_OUTPUT_ENABLE_DIS, true);
-        if (!ctx->setting<bool>("compress_rbf", false))
+        auto pn1 = CycloneV::pnode(bt, pos, port, bi, pi);
+        auto rn1 = cv->pnode_to_rnode(pn1);
+        if (rn1)
+            return rn1;
+
+        if (bt == CycloneV::GPIO) {
+            auto pn2 = cv->p2p_to(pn1);
+            if (!pn2) {
+                auto pnv = cv->p2p_from(pn1);
+                if (!pnv.empty())
+                    pn2 = pnv[0];
+            }
+            auto pn3 = cv->hmc_get_bypass(pn2);
+            auto rn2 = cv->pnode_to_rnode(pn3);
+            return rn2;
+        }
+
+        return 0;
+    }
+
+    void options()
+    {
+        if (!ctx->setting<bool>("compress_rbf", false)) {
             cv->opt_b_set(CycloneV::COMPRESSION_DIS, true);
-        cv->opt_n_set(CycloneV::CRC_DIVIDE_ORDER, 8);
-        cv->opt_b_set(CycloneV::CVP_CONF_DONE_EN, true);
-        cv->opt_b_set(CycloneV::DEVICE_WIDE_RESET_EN, true);
-        cv->opt_n_set(CycloneV::DRIVE_STRENGTH, 8);
-        cv->opt_b_set(CycloneV::IOCSR_READY_FROM_CSR_DONE_EN, true);
-        cv->opt_b_set(CycloneV::NCEO_DIS, true);
-        cv->opt_b_set(CycloneV::OCT_DONE_DIS, true);
-        cv->opt_r_set(CycloneV::OPT_A, 0x1dff);
-        if (!ctx->setting<bool>("compress_rbf", false))
             cv->opt_r_set(CycloneV::OPT_B, 0xffffff40adffffffULL);
-        else
+        } else
             cv->opt_r_set(CycloneV::OPT_B, 0xffffff402dffffffULL);
-        cv->opt_b_set(CycloneV::RELEASE_CLEARS_BEFORE_TRISTATES_DIS, true);
-        cv->opt_b_set(CycloneV::RETRY_CONFIG_ON_ERROR_EN, true);
-        cv->opt_r_set(CycloneV::START_UP_CLOCK, 0x3F);
-        // Default inversion
-        write_default_inv();
-    }
-
-    void write_default_inv()
-    {
-        // Some PNODEs are inverted by default. Set them up here.
-        for (const auto &pn2r : cv->get_all_p2r()) {
-            const auto &pn = pn2r.first;
-            auto pt = CycloneV::pn2pt(pn);
-            auto pi = CycloneV::pn2pi(pn);
-
-            switch (CycloneV::pn2bt(pn)) {
-            case CycloneV::HMC: {
-                // HMC OE are inverted to set OE=0, i.e. unused pins floating
-                // TODO: handle the case when we are using the HMC or HMC bypass
-                std::string name(CycloneV::port_type_names[pt]);
-                if (name.compare(0, 5, "IOINT") != 0 || name.compare(name.size() - 2, 2, "OE") != 0)
-                    continue;
-                cv->inv_set(pn2r.second, true);
-                break;
-            };
-            // HPS IO - TODO: what about when we actually support the HPS primitives?
-            case CycloneV::HPS_BOOT: {
-                switch (pt) {
-                case CycloneV::CSEL_EN:
-                case CycloneV::BSEL_EN:
-                case CycloneV::BOOT_FROM_FPGA_READY:
-                case CycloneV::BOOT_FROM_FPGA_ON_FAILURE:
-                    cv->inv_set(pn2r.second, true);
-                    break;
-                case CycloneV::CSEL:
-                    if (pi < 2)
-                        cv->inv_set(pn2r.second, true);
-                    break;
-                case CycloneV::BSEL:
-                    if (pi < 3)
-                        cv->inv_set(pn2r.second, true);
-                    break;
-                default:
-                    break;
-                };
-                break;
-            };
-            case CycloneV::HPS_CROSS_TRIGGER: {
-                if (pt == CycloneV::CLK_EN)
-                    cv->inv_set(pn2r.second, true);
-                break;
-            };
-            case CycloneV::HPS_TEST: {
-                if (pt == CycloneV::CFG_DFX_BYPASS_ENABLE)
-                    cv->inv_set(pn2r.second, true);
-                break;
-            };
-            case CycloneV::GPIO: {
-                // Ignore GPIO used by the design
-                BelId bel = ctx->bel_by_block_idx(CycloneV::pn2x(pn), CycloneV::pn2y(pn), id_MISTRAL_IO,
-                                                  CycloneV::pn2bi(pn));
-                if (bel != BelId() && ctx->getBoundBelCell(bel) != nullptr)
-                    continue;
-                // Bonded IO invert OEIN.1 which disables the output buffer and floats the IO
-                // Unbonded IO invert OEIN.0 which enables the output buffer, and {DATAIN.[0123]} to drive a constant
-                // GND, presumably for power/EMI reasons
-                bool is_bonded = cv->pin_find_pnode(pn) != nullptr;
-                if (is_bonded && (pt != CycloneV::OEIN || pi != 1))
-                    continue;
-                if (!is_bonded && (pt != CycloneV::DATAIN) && (pt != CycloneV::OEIN || pi != 0))
-                    continue;
-                cv->inv_set(pn2r.second, true);
-                break;
-            };
-            case CycloneV::FPLL: {
-                if (pt == CycloneV::EXTSWITCH || (pt == CycloneV::CLKEN && pi < 2))
-                    cv->inv_set(pn2r.second, true);
-                break;
-            };
-            default:
-                break;
-            }
-        }
-    }
-
-    void write_dqs()
-    {
-        for (auto pos : cv->dqs16_get_pos()) {
-            int x = CycloneV::pos2x(pos), y = CycloneV::pos2y(pos);
-            // DQS bypass for used output pins
-            for (int z = 0; z < 16; z++) {
-                int ioy = y + (z / 4) - 2;
-                if (ioy < 0 || ioy >= int(cv->get_tile_sy()))
-                    continue;
-                BelId bel = ctx->bel_by_block_idx(x, ioy, id_MISTRAL_IO, z % 4);
-                if (bel == BelId())
-                    continue;
-                CellInfo *ci = ctx->getBoundBelCell(bel);
-                if (ci == nullptr || (ci->type != id_MISTRAL_IO && ci->type != id_MISTRAL_OB))
-                    continue; // not an output
-                cv->bmux_m_set(CycloneV::DQS16, pos, CycloneV::INPUT_REG4_SEL, z, CycloneV::SEL_LOCKED_DPA);
-                cv->bmux_r_set(CycloneV::DQS16, pos, CycloneV::RB_T9_SEL_EREG_CFF_DELAY, z, 0x1f);
-            }
-        }
     }
 
     void write_routing()
@@ -174,30 +86,93 @@ struct MistralBitgen
 
     void write_io_cell(CellInfo *ci, int x, int y, int bi)
     {
-        bool is_output =
-                (ci->type == id_MISTRAL_OB || (ci->type == id_MISTRAL_IO && get_net_or_empty(ci, id_OE) != nullptr));
+        bool is_output = (ci->type == id_MISTRAL_OB || (ci->type == id_MISTRAL_IO && ci->getPort(id_OE) != nullptr));
         auto pos = CycloneV::xy2pos(x, y);
         // TODO: configurable pull, IO standard, etc
         cv->bmux_b_set(CycloneV::GPIO, pos, CycloneV::USE_WEAK_PULLUP, bi, false);
         if (is_output) {
             cv->bmux_m_set(CycloneV::GPIO, pos, CycloneV::DRIVE_STRENGTH, bi, CycloneV::V3P3_LVTTL_16MA_LVCMOS_2MA);
             cv->bmux_m_set(CycloneV::GPIO, pos, CycloneV::IOCSR_STD, bi, CycloneV::DIS);
+
+            // Output gpios must also bypass things in the associated dqs
+            auto dqs = cv->p2p_to(CycloneV::pnode(CycloneV::GPIO, pos, CycloneV::PNONE, bi, -1));
+            if (dqs) {
+                cv->bmux_m_set(CycloneV::DQS16, CycloneV::pn2p(dqs), CycloneV::INPUT_REG4_SEL, CycloneV::pn2bi(dqs),
+                               CycloneV::SEL_LOCKED_DPA);
+                cv->bmux_r_set(CycloneV::DQS16, CycloneV::pn2p(dqs), CycloneV::RB_T9_SEL_EREG_CFF_DELAY,
+                               CycloneV::pn2bi(dqs), 0x1f);
+            }
         }
         // There seem to be two mirrored OEIN inversion bits for constant OE for inputs/outputs. This might be to
         // prevent a single bitflip from turning inputs to outputs and messing up other devices on the boards, notably
         // ECP5 does similar. OEIN.0 inverted for outputs; OEIN.1 for inputs
-        cv->inv_set(cv->pnode_to_rnode(CycloneV::pnode(CycloneV::GPIO, pos, CycloneV::OEIN, bi, is_output ? 0 : 1)),
-                    true);
+        cv->inv_set(find_rnode(CycloneV::GPIO, pos, CycloneV::OEIN, bi, 0), is_output);
+        cv->inv_set(find_rnode(CycloneV::GPIO, pos, CycloneV::OEIN, bi, 1), !is_output);
     }
 
     void write_clkbuf_cell(CellInfo *ci, int x, int y, int bi)
     {
         (void)ci; // currently unused
         auto pos = CycloneV::xy2pos(x, y);
-        cv->bmux_r_set(CycloneV::CMUXHG, pos, CycloneV::INPUT_SELECT, bi, 0x1b); // hardcode to general routing
+        cv->bmux_r_set(CycloneV::CMUXHG, pos, CycloneV::INPUT_SEL, bi, 0x1b); // hardcode to general routing
         cv->bmux_m_set(CycloneV::CMUXHG, pos, CycloneV::TESTSYN_ENOUT_SELECT, bi, CycloneV::PRE_SYNENB);
     }
 
+    void write_m10k_cell(CellInfo *ci, int x, int y, int bi)
+    {
+        auto pos = CycloneV::xy2pos(x, y);
+
+        // Notes:
+        // DATA_FLOW_THRU is probably transparent reads.
+
+        cv->bmux_b_set(CycloneV::M10K, pos, CycloneV::A_DATA_FLOW_THRU, bi, 1);
+        cv->bmux_n_set(CycloneV::M10K, pos, CycloneV::A_DATA_WIDTH, bi, ci->params.at(id_CFG_DBITS).as_int64());
+        cv->bmux_m_set(CycloneV::M10K, pos, CycloneV::A_FAST_WRITE, bi, CycloneV::FAST);
+        cv->bmux_m_set(CycloneV::M10K, pos, CycloneV::A_OUTPUT_SEL, bi, CycloneV::ASYNC);
+        cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::A_SA_WREN_DELAY, bi, 1);
+        cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::A_SAEN_DELAY, bi, 2);
+        cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::A_WL_DELAY, bi, 2);
+        cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::A_WR_TIMER_PULSE, bi, 0x0b);
+
+        cv->bmux_b_set(CycloneV::M10K, pos, CycloneV::B_DATA_FLOW_THRU, bi, 1);
+        cv->bmux_n_set(CycloneV::M10K, pos, CycloneV::B_DATA_WIDTH, bi, ci->params.at(id_CFG_DBITS).as_int64());
+        cv->bmux_m_set(CycloneV::M10K, pos, CycloneV::B_FAST_WRITE, bi, CycloneV::FAST);
+        cv->bmux_m_set(CycloneV::M10K, pos, CycloneV::B_OUTPUT_SEL, bi, CycloneV::ASYNC);
+        cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::B_SA_WREN_DELAY, bi, 1);
+        cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::B_SAEN_DELAY, bi, 2);
+        cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::B_WL_DELAY, bi, 2);
+        cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::B_WR_TIMER_PULSE, bi, 0x0b);
+
+        cv->bmux_n_set(CycloneV::M10K, pos, CycloneV::TOP_CLK_SEL, bi, 1);
+        cv->bmux_b_set(CycloneV::M10K, pos, CycloneV::TOP_W_INV, bi, 1);
+        cv->bmux_n_set(CycloneV::M10K, pos, CycloneV::TOP_W_SEL, bi, 1);
+        cv->bmux_b_set(CycloneV::M10K, pos, CycloneV::BOT_CLK_INV, bi, 1);
+        cv->bmux_n_set(CycloneV::M10K, pos, CycloneV::BOT_W_SEL, bi, 1);
+
+        cv->bmux_b_set(CycloneV::M10K, pos, CycloneV::TRUE_DUAL_PORT, bi, 0);
+
+        cv->bmux_b_set(CycloneV::M10K, pos, CycloneV::DISABLE_UNUSED, bi, 0);
+
+        auto permute_init = [](int64_t init) -> int64_t {
+            const int permutation[40] = {0, 20, 10, 30, 1, 21, 11, 31, 2, 22, 12, 32, 3, 23, 13, 33, 4, 24, 14, 34,
+                                         5, 25, 15, 35, 6, 26, 16, 36, 7, 27, 17, 37, 8, 28, 18, 38, 9, 29, 19, 39};
+
+            int64_t output = 0;
+            for (int bit = 0; bit < 40; bit++)
+                output |= ((init >> permutation[bit]) & 1) << bit;
+            return ~output; // RAM init is inverted.
+        };
+
+        Property init;
+        if (ci->params.count(id_INIT) == 0) {
+            init = Property{0, 10240};
+        } else {
+            init = ci->params.at(id_INIT);
+        }
+        for (int bi = 0; bi < 256; bi++)
+            cv->bmux_r_set(CycloneV::M10K, pos, CycloneV::RAM, bi, permute_init(init.extract(bi * 40, 40).as_int64()));
+    }
+
     void write_cells()
     {
         for (auto &cell : ctx->cells) {
@@ -208,6 +183,8 @@ struct MistralBitgen
                 write_io_cell(ci, loc.x, loc.y, bi);
             else if (ctx->is_clkbuf_cell(ci->type))
                 write_clkbuf_cell(ci, loc.x, loc.y, bi);
+            else if (ci->type == id_MISTRAL_M10K)
+                write_m10k_cell(ci, loc.x, loc.y, bi);
         }
     }
 
@@ -233,7 +210,8 @@ struct MistralBitgen
         if (is_lutram) {
             for (int i = 0; i < 10; i++) {
                 // Many MLAB settings apply to the whole LAB, not just the ALM
-                cv->bmux_m_set(block_type, pos, CycloneV::MODE, i, CycloneV::RAM);
+                cv->bmux_m_set(block_type, pos, CycloneV::TMODE, i, CycloneV::RAM);
+                cv->bmux_m_set(block_type, pos, CycloneV::BMODE, i, CycloneV::RAM);
                 cv->bmux_n_set(block_type, pos, CycloneV::T_FEEDBACK_SEL, i, 1);
             }
             cv->bmux_r_set(block_type, pos, CycloneV::LUT_MASK, alm, 0xFFFFFFFFFFFFFFFFULL); // TODO: LUTRAM init
@@ -245,11 +223,11 @@ struct MistralBitgen
             cv->bmux_n_set(block_type, pos, CycloneV::WRITE_PULSE_LENGTH, 0, 650); // picoseconds, presumably
             // TODO: understand how these enables really work
             cv->bmux_b_set(block_type, pos, CycloneV::EN2_EN, 0, false);
-            cv->bmux_b_set(block_type, pos, CycloneV::EN_SCLK_LOAD_WHAT, 0, true);
-            cv->bmux_m_set(block_type, pos, CycloneV::SCLR_MUX, 0, CycloneV::GIN2);
+            cv->bmux_b_set(block_type, pos, CycloneV::SCLR_DIS, 0, true);
         } else {
-            // Combinational mode - TODO: flop feedback
-            cv->bmux_m_set(block_type, pos, CycloneV::MODE, alm, alm_data.l6_mode ? CycloneV::L6 : CycloneV::L5);
+            // Combinational mode - TODO: flop feedback and more modes...
+            cv->bmux_m_set(block_type, pos, CycloneV::TMODE, alm, alm_data.l6_mode ? CycloneV::C_E : CycloneV::E_0);
+            cv->bmux_m_set(block_type, pos, CycloneV::BMODE, alm, alm_data.l6_mode ? CycloneV::D_E : CycloneV::E_1);
             // LUT function
             cv->bmux_r_set(block_type, pos, CycloneV::LUT_MASK, alm, ctx->compute_lut_mask(lab, alm));
         }
@@ -308,8 +286,8 @@ struct MistralBitgen
             cv->bmux_m_set(block_type, pos, clk_sel[i / 2], alm, clk_choice[ce_idx]);
             if (ff->ffInfo.ctrlset.clk.inverted)
                 cv->bmux_b_set(block_type, pos, clk_inv[ce_idx], 0, true);
-            if (get_net_or_empty(ff, id_ENA) != nullptr) { // not using ffInfo.ctrlset, this has a fake net always to
-                                                           // ensure different constants don't collide
+            if (ff->getPort(id_ENA) != nullptr) { // not using ffInfo.ctrlset, this has a fake net always to
+                                                  // ensure different constants don't collide
                 cv->bmux_b_set(block_type, pos, en_en[ce_idx], 0, true);
                 cv->bmux_b_set(block_type, pos, en_ninv[ce_idx], 0, ff->ffInfo.ctrlset.ena.inverted);
             } else {
@@ -323,12 +301,17 @@ struct MistralBitgen
             // SCLR
             if (ff->ffInfo.ctrlset.sclr.net != nullptr) {
                 cv->bmux_b_set(block_type, pos, CycloneV::SCLR_INV, 0, ff->ffInfo.ctrlset.sclr.inverted);
+                cv->bmux_b_set(block_type, pos, CycloneV::SCLR_DIS, 0, false);
             } else {
                 cv->bmux_b_set(block_type, pos, sclr_dis[i / 2], alm, true);
             }
             // SLOAD
             if (ff->ffInfo.ctrlset.sload.net != nullptr) {
                 cv->bmux_b_set(block_type, pos, sload_en[i / 2], alm, true);
+                if (ff->ffInfo.ctrlset.sload.net->name == ctx->id("$PACKER_GND_NET")) {
+                    // force-disabled LOAD (see workaround in assign_ff_info)
+                    cv->bmux_b_set(block_type, pos, CycloneV::SLOAD_EN, 0, false);
+                }
                 cv->bmux_b_set(block_type, pos, CycloneV::SLOAD_INV, 0, ff->ffInfo.ctrlset.sload.inverted);
             }
         }
@@ -341,7 +324,7 @@ struct MistralBitgen
                 cv->bmux_m_set(block_type, pos, clk_sel[1], alm, clk_choice[ce_idx]);
                 if (lut->combInfo.wclk.inverted)
                     cv->bmux_b_set(block_type, pos, clk_inv[ce_idx], 0, true);
-                if (get_net_or_empty(lut, id_A1EN) != nullptr) {
+                if (lut->getPort(id_A1EN) != nullptr) {
                     cv->bmux_b_set(block_type, pos, en_en[ce_idx], 0, true);
                     cv->bmux_b_set(block_type, pos, en_ninv[ce_idx], 0, lut->combInfo.we.inverted);
                 } else {
@@ -363,18 +346,18 @@ struct MistralBitgen
 
         const std::array<CycloneV::bmux_type_t, 2> aclr_inp{CycloneV::ACLR0_SEL, CycloneV::ACLR1_SEL};
         for (int i = 0; i < 2; i++) {
-            // Quartus seems to set unused ACLRs to CLKI2...
-            if (!lab_data.aclr_used[i])
-                cv->bmux_m_set(block_type, pos, aclr_inp[i], 0, CycloneV::CLKI2);
-            else
-                cv->bmux_m_set(block_type, pos, aclr_inp[i], 0, (i == 1) ? CycloneV::GIN0 : CycloneV::GIN1);
+            // Quartus seems to set unused ACLRs to ACLR0
+            if (lab_data.aclr_used[i])
+                cv->bmux_m_set(block_type, pos, aclr_inp[i], 0, (i == 1) ? CycloneV::DIN2 : CycloneV::DIN3);
+            else if (i == 0)
+                cv->bmux_m_set(block_type, pos, aclr_inp[i], 0, CycloneV::ACLR0);
         }
         for (int i = 0; i < 3; i++) {
             // Check for fabric->clock routing
             if (ctx->wires_connected(
                         ctx->get_port(block_type, CycloneV::pos2x(pos), CycloneV::pos2y(pos), -1, CycloneV::DATAIN, 0),
                         lab_data.clk_wires[i]))
-                cv->bmux_m_set(block_type, pos, CycloneV::CLKA_SEL, 0, CycloneV::GIN2);
+                cv->bmux_m_set(block_type, pos, CycloneV::CLKA_SEL, 0, CycloneV::DIN0);
         }
     }
 
@@ -392,9 +375,8 @@ struct MistralBitgen
     void run()
     {
         cv->clear();
-        init();
+        options();
         write_routing();
-        write_dqs();
         write_cells();
         write_labs();
     }
