@@ -1072,7 +1072,7 @@ struct Arch : BaseArch<ArchRanges>
     bool getBelGlobalBuf(BelId bel) const override
     {
         IdString type = getBelType(bel);
-        return type == id_DCC || type == id_VCC_DRV;
+        return type.in(id_DCC, id_VCC_DRV);
     }
 
     IdString getBelType(BelId bel) const override
@@ -1522,6 +1522,8 @@ struct Arch : BaseArch<ArchRanges>
 
     // -------------------------------------------------
     void write_fasm(std::ostream &out) const;
+    // -------------------------------------------------
+    static void list_devices();
 };
 
 NEXTPNR_NAMESPACE_END
