@@ -533,11 +533,10 @@ struct NexusFasmWriter
     void write_dcc(const CellInfo *cell)
     {
         BelId bel = cell->bel;
-        push_tile(bel.tile);
-        push_belname(bel);
+        push_bel(bel);
         write_bit("DCCEN.1"); // Explicit DCC cell implies a clock buffer
         write_cell_muxes(cell);
-        pop(2);
+        pop();
     }
     // Write config for DCS
     void write_dcs(const CellInfo *cell)
@@ -596,7 +595,7 @@ struct NexusFasmWriter
         if (wid > 0) {
             push(stringf("IP_EBR_WID%d", wid));
             for (int i = 0; i < 64; i++) {
-                IdString param = ctx->id(stringf("INITVAL_%02X", i));
+                IdString param = ctx->idf("INITVAL_%02X", i);
                 if (!cell->params.count(param))
                     continue;
                 auto &prop = cell->params.at(param);
@@ -907,7 +906,7 @@ struct NexusFasmWriter
             l.x = 1;
         push(stringf("IP_LRAM_CORE_R%dC%d", l.y, l.x));
         for (int i = 0; i < 128; i++) {
-            IdString param = ctx->id(stringf("INITVAL_%02X", i));
+            IdString param = ctx->idf("INITVAL_%02X", i);
             if (!cell->params.count(param))
                 continue;
             auto &prop = cell->params.at(param);
@@ -1080,9 +1079,8 @@ struct NexusFasmWriter
                 write_osc(ci);
             else if (ci->type == id_OXIDE_EBR)
                 write_bram(ci);
-            else if (ci->type == id_MULT9_CORE || ci->type == id_PREADD9_CORE || ci->type == id_MULT18_CORE ||
-                     ci->type == id_MULT18X36_CORE || ci->type == id_MULT36_CORE || ci->type == id_REG18_CORE ||
-                     ci->type == id_ACC54_CORE)
+            else if (ci->type.in(id_MULT9_CORE, id_PREADD9_CORE, id_MULT18_CORE, id_MULT18X36_CORE, id_MULT36_CORE,
+                                 id_REG18_CORE, id_ACC54_CORE))
                 write_dsp(ci);
             else if (ci->type == id_PLL_CORE)
                 write_pll(ci);
@@ -1090,7 +1088,7 @@ struct NexusFasmWriter
                 write_lram(ci);
             else if (ci->type == id_DPHY_CORE)
                 write_dphy(ci);
-            else if (ci->type == id_IOLOGIC || ci->type == id_SIOLOGIC)
+            else if (ci->type.in(id_IOLOGIC, id_SIOLOGIC))
                 write_iol(ci);
             else if (ci->type == id_DCC)
                 write_dcc(ci);
